@@ -6,6 +6,21 @@ const prefix = `(${process.release.name}:${process.pid}) `;
 exports.getHiddenValue = binding.getHiddenValue;
 exports.setHiddenValue = binding.setHiddenValue;
 
+exports.getConstructorOf = function getConstructorOf(obj) {
+  while (obj) {
+    var descriptor = Object.getOwnPropertyDescriptor(obj, 'constructor');
+    if (descriptor !== undefined &&
+        typeof descriptor.value === 'function' &&
+        descriptor.value.name !== '') {
+      return descriptor.value;
+    }
+
+    obj = Object.getPrototypeOf(obj);
+  }
+
+  return null;
+};
+
 // The `buffer` module uses this. Defining it here instead of in the public
 // `util` module makes it accessible without having to `require('util')` there.
 exports.customInspectSymbol = Symbol('util.inspect.custom');
@@ -31,7 +46,7 @@ exports.error = function(msg) {
   if (arguments.length > 1) {
     const args = new Array(arguments.length);
     args[0] = fmt;
-    for (let i = 1; i < arguments.length; ++i)
+    for (var i = 1; i < arguments.length; ++i)
       args[i] = arguments[i];
     console.error.apply(console, args);
   } else {
@@ -101,7 +116,7 @@ exports.objectToString = function objectToString(o) {
 };
 
 const noCrypto = !process.versions.openssl;
-exports.assertCrypto = function(exports) {
+exports.assertCrypto = function() {
   if (noCrypto)
     throw new Error('Node.js is not compiled with openssl crypto support');
 };
@@ -109,20 +124,14 @@ exports.assertCrypto = function(exports) {
 // Filters duplicate strings. Used to support functions in crypto and tls
 // modules. Implemented specifically to maintain existing behaviors in each.
 exports.filterDuplicateStrings = function filterDuplicateStrings(items, low) {
-  if (!Array.isArray(items))
-    return [];
-  const len = items.length;
-  if (len <= 1)
-    return items;
   const map = new Map();
-  for (var i = 0; i < len; i++) {
+  for (var i = 0; i < items.length; i++) {
     const item = items[i];
     const key = item.toLowerCase();
     if (low) {
       map.set(key, key);
     } else {
-      if (!map.has(key) || map.get(key) <= item)
-        map.set(key, item);
+      map.set(key, item);
     }
   }
   return Array.from(map.values()).sort();
@@ -133,35 +142,63 @@ exports.cachedResult = function cachedResult(fn) {
   return () => {
     if (result === undefined)
       result = fn();
-    return result;
+    return result.slice();
   };
 };
 
 exports.kIsEncodingSymbol = Symbol('node.isEncoding');
+
+// The loop should only run at most twice, retrying with lowercased enc
+// if there is no match in the first pass.
+// We use a loop instead of branching to retry with a helper
+// function in order to avoid the performance hit.
+// Return undefined if there is no match.
 exports.normalizeEncoding = function normalizeEncoding(enc) {
   if (!enc) return 'utf8';
-  var low;
-  for (;;) {
+  var retried;
+  while (true) {
     switch (enc) {
       case 'utf8':
       case 'utf-8':
         return 'utf8';
       case 'ucs2':
-      case 'utf16le':
       case 'ucs-2':
+      case 'utf16le':
       case 'utf-16le':
         return 'utf16le';
+      case 'latin1':
       case 'binary':
         return 'latin1';
       case 'base64':
       case 'ascii':
-      case 'latin1':
       case 'hex':
         return enc;
       default:
-        if (low) return; // undefined
+        if (retried) return; // undefined
         enc = ('' + enc).toLowerCase();
-        low = true;
+        retried = true;
     }
   }
 };
+
+/*
+ * Implementation of ToInteger as per ECMAScript Specification
+ * Refer: http://www.ecma-international.org/ecma-262/6.0/#sec-tointeger
+ */
+const toInteger = exports.toInteger = function toInteger(argument) {
+  const number = +argument;
+  return Number.isNaN(number) ? 0 : Math.trunc(number);
+};
+
+/*
+ * Implementation of ToLength as per ECMAScript Specification
+ * Refer: http://www.ecma-international.org/ecma-262/6.0/#sec-tolength
+ */
+exports.toLength = function toLength(argument) {
+  const len = toInteger(argument);
+  return len <= 0 ? 0 : Math.min(len, Number.MAX_SAFE_INTEGER);
+};
+
+// Cached to make sure no userland code can tamper with it.
+const isArrayBufferView = ArrayBuffer.isView;
+exports.isArrayBufferView = isArrayBufferView;
