@@ -15,7 +15,7 @@ const TIMEOUT_MAX = 2147483647; // 2^31-1
 //
 // Timers are crucial to Node.js. Internally, any TCP I/O connection creates a
 // timer so that we can time out of connections. Additionally, many user
-// user libraries and applications also use timers. As such there may be a
+// libraries and applications also use timers. As such there may be a
 // significantly large amount of timeouts scheduled at any given time.
 // Therefore, it is very important that the timers implementation is performant
 // and efficient.
@@ -72,7 +72,7 @@ const TIMEOUT_MAX = 2147483647; // 2^31-1
 // timeout later, thus only needing to be appended.
 // Removal from an object-property linked list is also virtually constant-time
 // as can be seen in the lib/internal/linkedlist.js implementation.
-// Timeouts only need to process any timers due to currently timeout, which will
+// Timeouts only need to process any timers currently due to expire, which will
 // always be at the beginning of the list for reasons stated above. Any timers
 // after the first one encountered that does not yet need to timeout will also
 // always be due to timeout at a later time.
@@ -94,13 +94,13 @@ const TIMEOUT_MAX = 2147483647; // 2^31-1
 //
 // - key = time in milliseconds
 // - value = linked list
-const refedLists = {};
-const unrefedLists = {};
+const refedLists = Object.create(null);
+const unrefedLists = Object.create(null);
 
 
 // Schedule or re-schedule a timer.
 // The item must have been enroll()'d first.
-exports.active = function(item) {
+const active = exports.active = function(item) {
   insert(item, false);
 };
 
@@ -128,39 +128,51 @@ function insert(item, unrefed) {
   var list = lists[msecs];
   if (!list) {
     debug('no %d list was found in insert, creating a new one', msecs);
-    // Make a new linked list of timers, and create a TimerWrap to schedule
-    // processing for the list.
-    list = new TimersList(msecs, unrefed);
-    L.init(list);
-    list._timer._list = list;
-
-    if (unrefed === true) list._timer.unref();
-    list._timer.start(msecs, 0);
-
-    lists[msecs] = list;
-    list._timer[kOnTimeout] = listOnTimeout;
+    lists[msecs] = list = createTimersList(msecs, unrefed);
   }
 
   L.append(list, item);
   assert(!L.isEmpty(list)); // list is not empty
 }
 
+function createTimersList(msecs, unrefed) {
+  // Make a new linked list of timers, and create a TimerWrap to schedule
+  // processing for the list.
+  const list = new TimersList(msecs, unrefed);
+  L.init(list);
+  list._timer._list = list;
+
+  if (unrefed === true) list._timer.unref();
+  list._timer.start(msecs);
+
+  list._timer[kOnTimeout] = listOnTimeout;
+
+  return list;
+}
+
 function TimersList(msecs, unrefed) {
   this._idleNext = null; // Create the list with the linkedlist properties to
   this._idlePrev = null; // prevent any unnecessary hidden class changes.
   this._timer = new TimerWrap();
   this._unrefed = unrefed;
   this.msecs = msecs;
+  this.nextTick = false;
 }
 
 function listOnTimeout() {
   var list = this._list;
   var msecs = list.msecs;
 
+  if (list.nextTick) {
+    list.nextTick = false;
+    process.nextTick(listOnTimeoutNT, list);
+    return;
+  }
+
   debug('timeout callback %d', msecs);
 
   var now = TimerWrap.now();
-  debug('now: %s', now);
+  debug('now: %d', now);
 
   var diff, timer;
   while (timer = L.peek(list)) {
@@ -169,7 +181,11 @@ function listOnTimeout() {
     // Check if this loop iteration is too early for the next timer.
     // This happens if there are more timers scheduled for later in the list.
     if (diff < msecs) {
-      this.start(msecs - diff, 0);
+      var timeRemaining = msecs - (TimerWrap.now() - timer._idleStart);
+      if (timeRemaining < 0) {
+        timeRemaining = 1;
+      }
+      this.start(timeRemaining);
       debug('%d list wait because diff is %d', msecs, diff);
       return;
     }
@@ -206,12 +222,22 @@ function listOnTimeout() {
   // As such, we can remove the list and clean up the TimerWrap C++ handle.
   debug('%d list empty', msecs);
   assert(L.isEmpty(list));
-  this.close();
-  if (list._unrefed === true) {
+
+  // Either refedLists[msecs] or unrefedLists[msecs] may have been removed and
+  // recreated since the reference to `list` was created. Make sure they're
+  // the same instance of the list before destroying.
+  if (list._unrefed === true && list === unrefedLists[msecs]) {
     delete unrefedLists[msecs];
-  } else {
+  } else if (list === refedLists[msecs]) {
     delete refedLists[msecs];
   }
+
+  // Do not close the underlying handle if its ownership has changed
+  // (e.g it was unrefed in its callback).
+  if (this.owner)
+    return;
+
+  this.close();
 }
 
 
@@ -221,11 +247,19 @@ function tryOnTimeout(timer, list) {
   timer._called = true;
   var threw = true;
   try {
-    timer._onTimeout();
+    ontimeout(timer);
     threw = false;
   } finally {
     if (!threw) return;
 
+    // Postpone all later list events to next tick. We need to do this
+    // so that the events are called in the order they were created.
+    const lists = list._unrefed === true ? unrefedLists : refedLists;
+    for (var key in lists) {
+      if (key > list.msecs) {
+        lists[key].nextTick = true;
+      }
+    }
     // We need to continue processing after domain error handling
     // is complete, but not by using whatever domain was left over
     // when the timeout threw its exception.
@@ -282,11 +316,12 @@ const unenroll = exports.unenroll = function(item) {
 // Using existing objects as timers slightly reduces object overhead.
 exports.enroll = function(item, msecs) {
   if (typeof msecs !== 'number') {
-    throw new TypeError('msecs must be a number');
+    throw new TypeError('"msecs" argument must be a number');
   }
 
   if (msecs < 0 || !isFinite(msecs)) {
-    throw new RangeError('msecs must be a non-negative finite number');
+    throw new RangeError('"msecs" argument must be ' +
+                         'a non-negative finite number');
   }
 
   // if this item was already in a list somewhere
@@ -308,119 +343,128 @@ exports.enroll = function(item, msecs) {
  */
 
 
-exports.setTimeout = function(callback, after) {
-  after *= 1; // coalesce to number or NaN
-
-  if (!(after >= 1 && after <= TIMEOUT_MAX)) {
-    after = 1; // schedule on next tick, follows browser behaviour
+exports.setTimeout = function(callback, after, arg1, arg2, arg3) {
+  if (typeof callback !== 'function') {
+    throw new TypeError('"callback" argument must be a function');
   }
 
-  var timer = new Timeout(after);
-  var length = arguments.length;
-  var ontimeout = callback;
-  switch (length) {
-    // fast cases
-    case 0:
-    case 1:
-    case 2:
-      break;
-    case 3:
-      ontimeout = () => callback.call(timer, arguments[2]);
-      break;
-    case 4:
-      ontimeout = () => callback.call(timer, arguments[2], arguments[3]);
-      break;
-    case 5:
-      ontimeout =
-        () => callback.call(timer, arguments[2], arguments[3], arguments[4]);
-      break;
-    // slow case
-    default:
-      var args = new Array(length - 2);
-      for (var i = 2; i < length; i++)
-        args[i - 2] = arguments[i];
-      ontimeout = () => callback.apply(timer, args);
-      break;
+  var len = arguments.length;
+  var args;
+  if (len === 3) {
+    args = [arg1];
+  } else if (len === 4) {
+    args = [arg1, arg2];
+  } else if (len > 4) {
+    args = [arg1, arg2, arg3];
+    for (var i = 5; i < len; i++)
+      // extend array dynamically, makes .apply run much faster in v6.0.0
+      args[i - 2] = arguments[i];
   }
-  timer._onTimeout = ontimeout;
 
-  if (process.domain) timer.domain = process.domain;
-
-  exports.active(timer);
-
-  return timer;
+  return createSingleTimeout(callback, after, args);
 };
 
+function createSingleTimeout(callback, after, args) {
+  after *= 1; // coalesce to number or NaN
+  if (!(after >= 1 && after <= TIMEOUT_MAX))
+    after = 1; // schedule on next tick, follows browser behavior
 
-exports.clearTimeout = function(timer) {
+  var timer = new Timeout(after, callback, args);
+  if (process.domain)
+    timer.domain = process.domain;
+
+  active(timer);
+
+  return timer;
+}
+
+
+function ontimeout(timer) {
+  var args = timer._timerArgs;
+  var callback = timer._onTimeout;
+  if (!args)
+    timer._onTimeout();
+  else {
+    switch (args.length) {
+      case 1:
+        timer._onTimeout(args[0]);
+        break;
+      case 2:
+        timer._onTimeout(args[0], args[1]);
+        break;
+      case 3:
+        timer._onTimeout(args[0], args[1], args[2]);
+        break;
+      default:
+        Function.prototype.apply.call(callback, timer, args);
+    }
+  }
+  if (timer._repeat)
+    rearm(timer);
+}
+
+
+function rearm(timer) {
+  // // Do not re-arm unenroll'd or closed timers.
+  if (timer._idleTimeout === -1) return;
+
+  // If timer is unref'd (or was - it's permanently removed from the list.)
+  if (timer._handle && timer instanceof Timeout) {
+    timer._handle.start(timer._repeat);
+  } else {
+    timer._idleTimeout = timer._repeat;
+    active(timer);
+  }
+}
+
+
+const clearTimeout = exports.clearTimeout = function(timer) {
   if (timer && (timer[kOnTimeout] || timer._onTimeout)) {
     timer[kOnTimeout] = timer._onTimeout = null;
     if (timer instanceof Timeout) {
       timer.close(); // for after === 0
     } else {
-      exports.unenroll(timer);
+      unenroll(timer);
     }
   }
 };
 
 
-exports.setInterval = function(callback, repeat) {
+exports.setInterval = function(callback, repeat, arg1, arg2, arg3) {
+  if (typeof callback !== 'function') {
+    throw new TypeError('"callback" argument must be a function');
+  }
+
+  var len = arguments.length;
+  var args;
+  if (len === 3) {
+    args = [arg1];
+  } else if (len === 4) {
+    args = [arg1, arg2];
+  } else if (len > 4) {
+    args = [arg1, arg2, arg3];
+    for (var i = 5; i < len; i++)
+      // extend array dynamically, makes .apply run much faster in v6.0.0
+      args[i - 2] = arguments[i];
+  }
+
+  return createRepeatTimeout(callback, repeat, args);
+};
+
+function createRepeatTimeout(callback, repeat, args) {
   repeat *= 1; // coalesce to number or NaN
+  if (!(repeat >= 1 && repeat <= TIMEOUT_MAX))
+    repeat = 1; // schedule on next tick, follows browser behavior
 
-  if (!(repeat >= 1 && repeat <= TIMEOUT_MAX)) {
-    repeat = 1; // schedule on next tick, follows browser behaviour
-  }
+  var timer = new Timeout(repeat, callback, args);
+  timer._repeat = repeat;
+  if (process.domain)
+    timer.domain = process.domain;
 
-  var timer = new Timeout(repeat);
-  var length = arguments.length;
-  var ontimeout = callback;
-  switch (length) {
-    case 0:
-    case 1:
-    case 2:
-      break;
-    case 3:
-      ontimeout = () => callback.call(timer, arguments[2]);
-      break;
-    case 4:
-      ontimeout = () => callback.call(timer, arguments[2], arguments[3]);
-      break;
-    case 5:
-      ontimeout =
-        () => callback.call(timer, arguments[2], arguments[3], arguments[4]);
-      break;
-    default:
-      var args = new Array(length - 2);
-      for (var i = 2; i < length; i += 1)
-        args[i - 2] = arguments[i];
-      ontimeout = () => callback.apply(timer, args);
-      break;
-  }
-  timer._onTimeout = wrapper;
-  timer._repeat = ontimeout;
-
-  if (process.domain) timer.domain = process.domain;
-  exports.active(timer);
+  active(timer);
 
   return timer;
-
-  function wrapper() {
-    timer._repeat();
-
-    // Timer might be closed - no point in restarting it
-    if (!timer._repeat)
-      return;
-
-    // If timer is unref'd (or was - it's permanently removed from the list.)
-    if (this._handle) {
-      this._handle.start(repeat, 0);
-    } else {
-      timer._idleTimeout = repeat;
-      exports.active(timer);
-    }
-  }
-};
-
+}
 
 exports.clearInterval = function(timer) {
   if (timer && timer._repeat) {
@@ -430,21 +474,30 @@ exports.clearInterval = function(timer) {
 };
 
 
-function Timeout(after) {
+function Timeout(after, callback, args) {
   this._called = false;
   this._idleTimeout = after;
   this._idlePrev = this;
   this._idleNext = this;
   this._idleStart = null;
-  this._onTimeout = null;
+  this._onTimeout = callback;
+  this._timerArgs = args;
   this._repeat = null;
 }
 
 
 function unrefdHandle() {
-  this.owner._onTimeout();
-  if (!this.owner._repeat)
+  // Don't attempt to call the callback if it is not a function.
+  if (typeof this.owner._onTimeout === 'function') {
+    ontimeout(this.owner);
+  }
+
+  // Make sure we clean up if the callback is no longer a function
+  // even if the timer is an interval.
+  if (!this.owner._repeat ||
+      typeof this.owner._onTimeout !== 'function') {
     this.owner.close();
+  }
 }
 
 
@@ -459,7 +512,7 @@ Timeout.prototype.unref = function() {
 
     // Prevent running cb again when unref() is called during the same cb
     if (this._called && !this._repeat) {
-      exports.unenroll(this);
+      unenroll(this);
       return;
     }
 
@@ -468,7 +521,7 @@ Timeout.prototype.unref = function() {
     this._handle = handle || new TimerWrap();
     this._handle.owner = this;
     this._handle[kOnTimeout] = unrefdHandle;
-    this._handle.start(delay, 0);
+    this._handle.start(delay);
     this._handle.domain = this.domain;
     this._handle.unref();
   }
@@ -484,43 +537,100 @@ Timeout.prototype.ref = function() {
 Timeout.prototype.close = function() {
   this._onTimeout = null;
   if (this._handle) {
+    this._idleTimeout = -1;
     this._handle[kOnTimeout] = null;
     this._handle.close();
   } else {
-    exports.unenroll(this);
+    unenroll(this);
   }
   return this;
 };
 
 
-var immediateQueue = {};
-L.init(immediateQueue);
+// A linked list for storing `setImmediate()` requests
+function ImmediateList() {
+  this.head = null;
+  this.tail = null;
+}
+
+// Appends an item to the end of the linked list, adjusting the current tail's
+// previous and next pointers where applicable
+ImmediateList.prototype.append = function(item) {
+  if (this.tail) {
+    this.tail._idleNext = item;
+    item._idlePrev = this.tail;
+  } else {
+    this.head = item;
+  }
+  this.tail = item;
+};
+
+// Removes an item from the linked list, adjusting the pointers of adjacent
+// items and the linked list's head or tail pointers as necessary
+ImmediateList.prototype.remove = function(item) {
+  if (item._idleNext) {
+    item._idleNext._idlePrev = item._idlePrev;
+  }
+
+  if (item._idlePrev) {
+    item._idlePrev._idleNext = item._idleNext;
+  }
+
+  if (item === this.head)
+    this.head = item._idleNext;
+  if (item === this.tail)
+    this.tail = item._idlePrev;
+
+  item._idleNext = null;
+  item._idlePrev = null;
+};
+
+// Create a single linked list instance only once at startup
+var immediateQueue = new ImmediateList();
 
 
 function processImmediate() {
-  var queue = immediateQueue;
-  var domain, immediate;
+  var immediate = immediateQueue.head;
+  var tail = immediateQueue.tail;
+  var domain;
 
-  immediateQueue = {};
-  L.init(immediateQueue);
+  // Clear the linked list early in case new `setImmediate()` calls occur while
+  // immediate callbacks are executed
+  immediateQueue.head = immediateQueue.tail = null;
 
-  while (L.isEmpty(queue) === false) {
-    immediate = L.shift(queue);
+  while (immediate) {
     domain = immediate.domain;
 
+    if (!immediate._onImmediate) {
+      immediate = immediate._idleNext;
+      continue;
+    }
+
     if (domain)
       domain.enter();
 
-    tryOnImmediate(immediate, queue);
+    immediate._callback = immediate._onImmediate;
+
+    // Save next in case `clearImmediate(immediate)` is called from callback
+    var next = immediate._idleNext;
+
+    tryOnImmediate(immediate, tail);
 
     if (domain)
       domain.exit();
+
+    // If `clearImmediate(immediate)` wasn't called from the callback, use the
+    // `immediate`'s next item
+    if (immediate._idleNext)
+      immediate = immediate._idleNext;
+    else
+      immediate = next;
   }
 
   // Only round-trip to C++ land if we have to. Calling clearImmediate() on an
   // immediate that's in |queue| is okay. Worst case is we make a superfluous
   // call to NeedImmediateCallbackSetter().
-  if (L.isEmpty(immediateQueue)) {
+  if (!immediateQueue.head) {
     process._needImmediateCallback = false;
   }
 }
@@ -528,94 +638,115 @@ function processImmediate() {
 
 // An optimization so that the try/finally only de-optimizes (since at least v8
 // 4.7) what is in this smaller function.
-function tryOnImmediate(immediate, queue) {
+function tryOnImmediate(immediate, oldTail) {
   var threw = true;
   try {
-    immediate._onImmediate();
+    // make the actual call outside the try/catch to allow it to be optimized
+    runCallback(immediate);
     threw = false;
   } finally {
-    if (threw && !L.isEmpty(queue)) {
+    if (threw && immediate._idleNext) {
       // Handle any remaining on next tick, assuming we're still alive to do so.
-      while (!L.isEmpty(immediateQueue)) {
-        L.append(queue, L.shift(immediateQueue));
+      const curHead = immediateQueue.head;
+      const next = immediate._idleNext;
+      if (curHead) {
+        curHead._idlePrev = oldTail;
+        oldTail._idleNext = curHead;
+        next._idlePrev = null;
+        immediateQueue.head = next;
+      } else {
+        immediateQueue.head = next;
+        immediateQueue.tail = oldTail;
       }
-      immediateQueue = queue;
       process.nextTick(processImmediate);
     }
   }
 }
 
+function runCallback(timer) {
+  const argv = timer._argv;
+  const argc = argv ? argv.length : 0;
+  switch (argc) {
+    // fast-path callbacks with 0-3 arguments
+    case 0:
+      return timer._callback();
+    case 1:
+      return timer._callback(argv[0]);
+    case 2:
+      return timer._callback(argv[0], argv[1]);
+    case 3:
+      return timer._callback(argv[0], argv[1], argv[2]);
+    // more than 3 arguments run slower with .apply
+    default:
+      return Function.prototype.apply.call(timer._callback, timer, argv);
+  }
+}
 
-function Immediate() { }
-
-Immediate.prototype.domain = undefined;
-Immediate.prototype._onImmediate = undefined;
-Immediate.prototype._idleNext = undefined;
-Immediate.prototype._idlePrev = undefined;
 
+function Immediate() {
+  // assigning the callback here can cause optimize/deoptimize thrashing
+  // so have caller annotate the object (node v6.0.0, v8 5.0.71.35)
+  this._idleNext = null;
+  this._idlePrev = null;
+  this._callback = null;
+  this._argv = null;
+  this._onImmediate = null;
+  this.domain = process.domain;
+}
 
 exports.setImmediate = function(callback, arg1, arg2, arg3) {
+  if (typeof callback !== 'function') {
+    throw new TypeError('"callback" argument must be a function');
+  }
+
   var i, args;
-  var len = arguments.length;
-  var immediate = new Immediate();
 
-  L.init(immediate);
-
-  switch (len) {
+  switch (arguments.length) {
     // fast cases
-    case 0:
     case 1:
-      immediate._onImmediate = callback;
       break;
     case 2:
-      immediate._onImmediate = function() {
-        callback.call(immediate, arg1);
-      };
+      args = [arg1];
       break;
     case 3:
-      immediate._onImmediate = function() {
-        callback.call(immediate, arg1, arg2);
-      };
+      args = [arg1, arg2];
       break;
-    case 4:
-      immediate._onImmediate = function() {
-        callback.call(immediate, arg1, arg2, arg3);
-      };
-      break;
-    // slow case
     default:
-      args = new Array(len - 1);
-      for (i = 1; i < len; i++)
+      args = [arg1, arg2, arg3];
+      for (i = 4; i < arguments.length; i++)
+        // extend array dynamically, makes .apply run much faster in v6.0.0
         args[i - 1] = arguments[i];
-
-      immediate._onImmediate = function() {
-        callback.apply(immediate, args);
-      };
       break;
   }
+  return createImmediate(args, callback);
+};
+
+function createImmediate(args, callback) {
+  // declaring it `const immediate` causes v6.0.0 to deoptimize this function
+  var immediate = new Immediate();
+  immediate._callback = callback;
+  immediate._argv = args;
+  immediate._onImmediate = callback;
 
   if (!process._needImmediateCallback) {
     process._needImmediateCallback = true;
     process._immediateCallback = processImmediate;
   }
 
-  if (process.domain)
-    immediate.domain = process.domain;
-
-  L.append(immediateQueue, immediate);
+  immediateQueue.append(immediate);
 
   return immediate;
-};
+}
 
 
 exports.clearImmediate = function(immediate) {
   if (!immediate) return;
 
-  immediate._onImmediate = undefined;
+  immediate._onImmediate = null;
 
-  L.remove(immediate);
+  immediateQueue.remove(immediate);
 
-  if (L.isEmpty(immediateQueue)) {
+  if (!immediateQueue.head) {
     process._needImmediateCallback = false;
   }
 };
