@@ -8,10 +8,9 @@ const inherits = util.inherits;
 const errnoException = util._errnoException;
 const readline = require('readline');
 
-
-exports.isatty = function(fd) {
-  return isTTY(fd);
-};
+function isatty(fd) {
+  return Number.isInteger(fd) && fd >= 0 && isTTY(fd);
+}
 
 
 function ReadStream(fd, options) {
@@ -32,8 +31,6 @@ function ReadStream(fd, options) {
 }
 inherits(ReadStream, net.Socket);
 
-exports.ReadStream = ReadStream;
-
 ReadStream.prototype.setRawMode = function(flag) {
   flag = !!flag;
   this._handle.setRawMode(flag);
@@ -64,7 +61,6 @@ function WriteStream(fd) {
   }
 }
 inherits(WriteStream, net.Socket);
-exports.WriteStream = WriteStream;
 
 
 WriteStream.prototype.isTTY = true;
@@ -105,3 +101,6 @@ WriteStream.prototype.clearScreenDown = function() {
 WriteStream.prototype.getWindowSize = function() {
   return [this.columns, this.rows];
 };
+
+
+module.exports = { isatty, ReadStream, WriteStream };
