@@ -1,9 +1,7 @@
-#include "async-wrap.h"
 #include "async-wrap-inl.h"
 #include "env.h"
 #include "env-inl.h"
 #include "handle_wrap.h"
-#include "util.h"
 #include "util-inl.h"
 
 #include <stdint.h>
@@ -11,7 +9,6 @@
 namespace node {
 
 using v8::Context;
-using v8::Function;
 using v8::FunctionCallbackInfo;
 using v8::FunctionTemplate;
 using v8::HandleScope;
@@ -75,8 +72,7 @@ class TimerWrap : public HandleWrap {
     CHECK(HandleWrap::IsAlive(wrap));
 
     int64_t timeout = args[0]->IntegerValue();
-    int64_t repeat = args[1]->IntegerValue();
-    int err = uv_timer_start(&wrap->handle_, OnTimeout, timeout, repeat);
+    int err = uv_timer_start(&wrap->handle_, OnTimeout, timeout, 0);
     args.GetReturnValue().Set(err);
   }
 
