@@ -1,8 +1,8 @@
 'use strict';
+const common = require('../common');
 const path = require('path');
 const assert = require('assert');
 const spawn = require('child_process').spawn;
-const common = require('../common');
 const debug = require('_debugger');
 
 const scenarios = [];
@@ -73,6 +73,7 @@ function runScenario(scriptName, throwsOnLine, next) {
     client.connect(port);
   }
 
+  let interval;
   function runTest(client) {
     client.req(
       {
@@ -91,19 +92,22 @@ function runScenario(scriptName, throwsOnLine, next) {
 
         client.reqContinue(function(error) {
           assert.ifError(error);
-          setTimeout(assertHasPaused.bind(null, client), 100);
+          interval = setInterval(assertHasPaused.bind(null, client), 10);
         });
       }
     );
   }
 
   function assertHasPaused(client) {
-    assert(exceptions.length, 'no exceptions thrown, race condition in test?');
-    assert.equal(exceptions.length, 1, 'debugger did not pause on exception');
-    assert.equal(exceptions[0].uncaught, true);
-    assert.equal(exceptions[0].script.name, testScript);
-    assert.equal(exceptions[0].sourceLine + 1, throwsOnLine);
+    if (!exceptions.length) return;
+
+    assert.strictEqual(exceptions.length, 1,
+                       'debugger did not pause on exception');
+    assert.strictEqual(exceptions[0].uncaught, true);
+    assert.strictEqual(exceptions[0].script.name, testScript);
+    assert.strictEqual(exceptions[0].sourceLine + 1, throwsOnLine);
     asserted = true;
     client.reqContinue(assert.ifError);
+    clearInterval(interval);
   }
 }
