@@ -1,11 +1,11 @@
 'use strict';
 require('../common');
-var assert = require('assert');
-var http = require('http');
+const assert = require('assert');
+const http = require('http');
 
-var testResBody = 'other stuff!\n';
+const testResBody = 'other stuff!\n';
 
-var server = http.createServer(function(req, res) {
+const server = http.createServer((req, res) => {
   assert.ok(!('date' in req.headers),
             'Request headers contained a Date.');
   res.writeHead(200, {
@@ -16,16 +16,16 @@ var server = http.createServer(function(req, res) {
 server.listen(0);
 
 
-server.addListener('listening', function() {
-  var options = {
-    port: this.address().port,
+server.addListener('listening', () => {
+  const options = {
+    port: server.address().port,
     path: '/',
     method: 'GET'
   };
-  var req = http.request(options, function(res) {
+  const req = http.request(options, (res) => {
     assert.ok('date' in res.headers,
               'Response headers didn\'t contain a Date.');
-    res.addListener('end', function() {
+    res.addListener('end', () => {
       server.close();
       process.exit();
     });
