@@ -1,13 +1,13 @@
 'use strict';
-var common = require('../common');
-var assert = require('assert');
+require('../common');
+const assert = require('assert');
 
 // calling .exit() from within "exit" should not overflow the call stack
-var nexits = 0;
+let nexits = 0;
 
 process.on('exit', function(code) {
-  assert.equal(nexits++, 0);
-  assert.equal(code, 0);
+  assert.strictEqual(nexits++, 0);
+  assert.strictEqual(code, 0);
   process.exit();
 });
 
