@@ -1,23 +1,21 @@
 'use strict';
-require('../common');
-var PassThrough = require('stream').PassThrough;
-var assert = require('assert');
-var inherits = require('util').inherits;
-var extend = require('util')._extend;
-var Interface = require('readline').Interface;
+const common = require('../common');
+const PassThrough = require('stream').PassThrough;
+const assert = require('assert');
+const Interface = require('readline').Interface;
 
+class FakeInput extends PassThrough {}
 
-function FakeInput() {
-  PassThrough.call(this);
+function extend(k) {
+  return Object.assign({ ctrl: false, meta: false, shift: false }, k);
 }
-inherits(FakeInput, PassThrough);
 
 
-var fi = new FakeInput();
-var fo = new FakeInput();
+const fi = new FakeInput();
+const fo = new FakeInput();
 new Interface({ input: fi, output: fo, terminal: true });
 
-var keys = [];
+let keys = [];
 fi.on('keypress', function(s, k) {
   keys.push(k);
 });
@@ -32,9 +30,7 @@ function addTest(sequences, expectedKeys) {
     expectedKeys = [ expectedKeys ];
   }
 
-  expectedKeys = expectedKeys.map(function(k) {
-    return k ? extend({ ctrl: false, meta: false, shift: false }, k) : k;
-  });
+  expectedKeys = expectedKeys.map(extend);
 
   keys = [];
 
@@ -55,7 +51,7 @@ function addTest(sequences, expectedKeys) {
 
 const addKeyIntervalTest = (sequences, expectedKeys, interval = 550,
                             assertDelay = 550) => {
-  return (next) => () => {
+  const fn = common.mustCall((next) => () => {
 
     if (!Array.isArray(sequences)) {
       sequences = [ sequences ];
@@ -65,9 +61,7 @@ const addKeyIntervalTest = (sequences, expectedKeys, interval = 550,
       expectedKeys = [ expectedKeys ];
     }
 
-    expectedKeys = expectedKeys.map(function(k) {
-      return k ? extend({ ctrl: false, meta: false, shift: false }, k) : k;
-    });
+    expectedKeys = expectedKeys.map(extend);
 
     const keys = [];
     fi.on('keypress', (s, k) => keys.push(k));
@@ -84,7 +78,8 @@ const addKeyIntervalTest = (sequences, expectedKeys, interval = 550,
       }
     };
     emitKeys(sequences);
-  };
+  });
+  return fn;
 };
 
 // regular alphanumerics
