@@ -23,14 +23,14 @@ r.defineCommand('say1', {
   help: 'help for say1',
   action: function(thing) {
     output = '';
-    this.write(`hello ${thing}`);
+    this.output.write(`hello ${thing}\n`);
     this.displayPrompt();
   }
 });
 
 r.defineCommand('say2', function() {
   output = '';
-  this.write('hello from say2');
+  this.output.write('hello from say2\n');
   this.displayPrompt();
 });
 
@@ -38,6 +38,12 @@ inputStream.write('.help\n');
 assert(/\n.say1     help for say1\n/.test(output), 'help for say1 not present');
 assert(/\n.say2\n/.test(output), 'help for say2 not present');
 inputStream.write('.say1 node developer\n');
-assert(/> hello node developer/.test(output), 'say1 outputted incorrectly');
+assert.ok(output.startsWith('hello node developer\n'),
+          `say1 output starts incorrectly: "${output}"`);
+assert.ok(output.includes('> '),
+          `say1 output does not include prompt: "${output}"`);
 inputStream.write('.say2 node developer\n');
-assert(/> hello from say2/.test(output), 'say2 outputted incorrectly');
+assert.ok(output.startsWith('hello from say2\n'),
+          `say2 output starts incorrectly: "${output}"`);
+assert.ok(output.includes('> '),
+          `say2 output does not include prompt: "${output}"`);
