@@ -3,9 +3,8 @@
 const common = require('../common');
 
 // FIXME add sunos support
-if (!(common.isFreeBSD || common.isOSX || common.isLinux)) {
-  return common.skip(`Unsupported platform [${process.platform}]`);
-}
+if (!(common.isFreeBSD || common.isOSX || common.isLinux))
+  common.skip(`Unsupported platform [${process.platform}]`);
 
 const assert = require('assert');
 const exec = require('child_process').exec;
@@ -23,15 +22,15 @@ assert.strictEqual(process.title, title);
 // To pass this test on alpine, since Busybox `ps` does not
 // support `-p` switch, use `ps -o` and `grep` instead.
 const cmd = common.isLinux ?
-            `ps -o pid,args | grep '${process.pid} ${title}' | grep -v grep` :
-            `ps -p ${process.pid} -o args=`;
+  `ps -o pid,args | grep '${process.pid} ${title}' | grep -v grep` :
+  `ps -p ${process.pid} -o args=`;
 
 exec(cmd, common.mustCall((error, stdout, stderr) => {
   assert.ifError(error);
   assert.strictEqual(stderr, '');
 
   // freebsd always add ' (procname)' to the process title
-  if (common.isFreeBSD)
+  if (common.isFreeBSD || common.isOpenBSD)
     title += ` (${path.basename(process.execPath)})`;
 
   // omitting trailing whitespace and \n
