@@ -37,7 +37,7 @@ if (!process.argv[2]) {
 
   const comspec = process.env['comspec'];
   if (!comspec || comspec.length === 0) {
-    common.fail('Failed to get COMSPEC');
+    assert.fail('Failed to get COMSPEC');
   }
 
   const args = ['/c', process.execPath, __filename, 'child',
