@@ -1,19 +1,15 @@
 'use strict';
 const common = require('../common');
-const assert = require('assert');
-
-if (!common.hasCrypto) {
+if (!common.hasCrypto)
   common.skip('missing crypto');
-  return;
-}
-const tls = require('tls');
 
-const fs = require('fs');
-const path = require('path');
+const assert = require('assert');
+const tls = require('tls');
+const fixtures = require('../common/fixtures');
 
 const options = {
-  key: fs.readFileSync(path.join(common.fixturesDir, 'test_key.pem')),
-  cert: fs.readFileSync(path.join(common.fixturesDir, 'test_cert.pem'))
+  key: fixtures.readSync('test_key.pem'),
+  cert: fixtures.readSync('test_cert.pem')
 };
 
 const server = tls.createServer(options, common.mustCall(function(socket) {
@@ -35,14 +31,14 @@ function unauthorized() {
     socket.end();
     rejectUnauthorized();
   }));
-  socket.on('error', common.fail);
+  socket.on('error', common.mustNotCall());
   socket.write('ok');
 }
 
 function rejectUnauthorized() {
   const socket = tls.connect(server.address().port, {
     servername: 'localhost'
-  }, common.fail);
+  }, common.mustNotCall());
   socket.on('error', common.mustCall(function(err) {
     console.error(err);
     authorized();
@@ -52,13 +48,13 @@ function rejectUnauthorized() {
 
 function authorized() {
   const socket = tls.connect(server.address().port, {
-    ca: [fs.readFileSync(path.join(common.fixturesDir, 'test_cert.pem'))],
+    ca: [fixtures.readSync('test_cert.pem')],
     servername: 'localhost'
   }, common.mustCall(function() {
     assert(socket.authorized);
     socket.end();
     server.close();
   }));
-  socket.on('error', common.fail);
+  socket.on('error', common.mustNotCall());
   socket.write('ok');
 }
